<?php

use Zen\Payment\Notification;
use Zen\Payment\Util;

include_once(Mage::getConfig()->getModuleDir('model', 'Zen') . '/Helper/payment-core/vendor/autoload.php');
include_once(Mage::getConfig()->getModuleDir('model', 'Zen') . '/Helper/payment-core/src/Util.php');
include_once(Mage::getConfig()->getModuleDir('model', 'Zen') . '/Helper/payment-core/src/Notification.php');

/**
 * Class Zen_NotificationController
 */
class Zen_NotificationController extends Mage_Core_Controller_Front_Action
{

	/**
	 * Verify notification and change status
	 *
	 * @throws Exception
	 */
	public function processAction()
	{

		$notification = new Notification(
			Mage::getStoreConfig('payment/zen/ipn_secret')
		);

		// it can be order data or notification code - depends on verification notification
		$resultCheckRequestNotification = $notification->checkRequest();

		if(is_int($resultCheckRequestNotification)) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit();
		}

		$transactionStatus = $resultCheckRequestNotification['status'];

		if($resultCheckRequestNotification['type'] === \Zen\Payment\Notification::TRT_REFUND) {

			if(!($order = Mage::getModel("sales/order")->loadByIncrementId(
				Util::extractMerchantTransactionId($resultCheckRequestNotification['merchantTransactionId'])
			))) {

				echo Notification::formatResponse(Notification::NS_ERROR);
				exit();
			}

			if($transactionStatus !== Notification::STATUS_ACCEPTED) {
				echo Notification::formatResponse(Notification::NS_OK);
				exit();
			}

			if(!$order->canCreditmemo()) {

				echo \Zen\Payment\Notification::formatResponse(\Zen\Payment\Notification::NS_ERROR);
				exit();
			}

			try {

				$data = array();
				$refundToStoreCreditAmount = $resultCheckRequestNotification['amount'];

				$service = Mage::getModel('sales/service_order', $order);

				$creditmemo = $service->prepareCreditmemo($data);

				if($refundToStoreCreditAmount) {
					// check if refund to Store Credit is available

					$refundToStoreCreditAmount = max(
						0,
						min($creditmemo->getBaseCustomerBalanceReturnMax(), $refundToStoreCreditAmount)
					);

					if($refundToStoreCreditAmount) {
						$refundToStoreCreditAmount = $creditmemo->getStore()->roundPrice($refundToStoreCreditAmount);

						$creditmemo->setBaseCustomerBalanceTotalRefunded($refundToStoreCreditAmount);
						$refundToStoreCreditAmount = $creditmemo->getStore()->roundPrice(
							$refundToStoreCreditAmount * $order->getStoreToOrderRate()
						);
						// this field can be used by customer balance observer
						$creditmemo->setBsCustomerBalTotalRefunded($refundToStoreCreditAmount);
						// setting flag to make actual refund to customer balance after credit memo save
						$creditmemo->setCustomerBalanceRefundFlag(true);
					}
				}

				$creditmemo->setPaymentRefundDisallowed(true);

				$creditmemo->register();

				$orderCreditMemoStatusCode = Mage_Sales_Model_Order::STATE_CLOSED;
				$orderCreditMemoStatusComment = 'Order Refunded.';
				Mage::getModel('core/resource_transaction')->addObject($creditmemo)->addObject($order)->save();
				$order->addStatusToHistory($orderCreditMemoStatusCode, $orderCreditMemoStatusComment, true);

				$notifyCustomer = true;
				$comment = 'Refund via ZEN api. Amount: '
					. $refundToStoreCreditAmount
					. ' '
					. $resultCheckRequestNotification['currency']
					. ', transaction ID: ' . $resultCheckRequestNotification['transactionId'];
				$creditmemo->setEmailSent(true);

				if(!empty($comment)) {
					$creditmemo->addComment($comment, $notifyCustomer);
				}

				try {
					Mage::getModel('core/resource_transaction')
						->addObject($creditmemo)
						->addObject($order)
						->save();

					$creditmemo->sendEmail($notifyCustomer);
				} catch(Mage_Core_Exception $e) {

				}
			} catch(\Exception $e) {

				echo \Zen\Payment\Notification::formatResponse(\Zen\Payment\Notification::NS_ERROR);
				exit();
			}

			echo \Zen\Payment\Notification::formatResponse(\Zen\Payment\Notification::NS_OK);
			exit();
		}

		if(!($order = Mage::getModel("sales/order")->loadByIncrementId($resultCheckRequestNotification['merchantTransactionId']))) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit();
		}

		if($order->getState() === Mage_Sales_Model_Order::STATE_PROCESSING) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit();
		}

		if(!Notification::checkRequestAmount($resultCheckRequestNotification, $order->getGrandTotal(), $order->getOrderCurrencyCode())) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit();
		}

		$payment = $order->getPayment();
		$transactionReference = $resultCheckRequestNotification['transactionId'];

		// Handle transaction states
		switch($transactionStatus) {
			case Notification::STATUS_ACCEPTED:
				$payment->setTransactionId($transactionReference)
					->setPreparedMessage(Mage::helper('core')->__('The transaction completed successfully'))
					->setCurrencyCode($payment->getOrder()->getBaseCurrencyCode())
					->setIsTransactionApproved(true)
					->setIsTransactionClosed(true)
					->registerCaptureNotification($order->getTotalDue(), true)
					->save();

				$order->save();

				if($invoice = $payment->getCreatedInvoice()) {
					$comment = Mage::helper('core')->__('Notified customer about invoice #%s.', $invoice->getIncrementId());
					$order->queueNewOrderEmail()
						->addStatusHistoryComment($comment)
						->setIsCustomerNotified(true)
						->save();
				}
				echo Notification::formatResponse(Notification::NS_OK);
				die();
			case Notification::STATUS_REJECTED:
				$comment = Mage::helper('core')->__('The payment has been rejected.');

				$payment->setTransactionId($transactionReference)
					->setPreparedMessage($comment)
					->save();

				$order->setState(Mage_Sales_Model_Order::STATE_PAYMENT_REVIEW, true, $comment)
					->save();
				echo Notification::formatResponse(Notification::NS_OK);
				die();
			default:
				echo Notification::formatResponse(Notification::NS_OK);
				exit;
		}
	}
}
